
/*************************************************************************************
 * FUNCTION: main
 * 
 * DESCRIPTION: This main module is used to dump the hdfeos object to binary format.
 *
 * The program retrieves the user input options for the type of delimiter to be used, 
 * number of unit data for grid and swath, or number of fields for point object to 
 * be printed in a line,  the level number (for point object only),
 * the field name, the file name from which the object is to be extracted from the 
 * command line  shown below:
 * 
 * <program_name>  <([-l level_number] [-f field_name]) | [-all]> 
 *              <object_name>  <file_name>  <binary_file>
 *
 *     command line Input       type         description
 *     ==================       =========    =================================
 *
 *     -l level_number          optional     level for point object only. valid value 
 *                                           is 0-7. exclusive with -all option
 *     -f fieldname             optional     the fieldname of a hdfeos object.
 *                                            exclusive with -all option
 *                                           
 *     -all                     optional     all fields will be dumped. exclusive 
 *                                            with -l and -f options.
 *                                          
 *
 *                                           number is 10.
 *     object_name              required     hdfeos object name. 
 *     filename                 required     hdfeos filename           
 *     binary_file              required     output file for binary dump.
 *
 *     NOTE:  both options "-l" and "-f" are exclusive with "-all". 
 *            But either one must be specified.
 *           
 *
 *
 * If error occurs in this main function, a value of -1 is returned and the program
 * writes an error message to stderr. Otherwise 0 is returned.         
 *
 * INPUTS: 
 *         Name     Type        Description
 *         =====    =====      ===================================
 *         argc     int         No of arguments in command line
 *         argv[]   char *      array of command line arguments
 * 
 * OUTPUTS:
 *         None
 *
 * RETURN VALUE: 
 *        status     int        0   SUCCESSED 
 *                             -1   FAIL
 * Author:  Subing Zeng     12/01/98       Raytheon STX Corp.
 * Modified : Urmila Prasad 02/10/99       Raytheon STX Corp.
 * Modified : Urmila Prasad 10/28/99 (Fixed the -all problem ) Raytheon Corp
 *
 * Modified asciidmp.c to get binary dump to an output file.
 **************************************************************************************/

#include "bindmp.h"

/* global variables */
int col_in_a_line = 10;		/* number of unit data to be printed in a

				 * line */
char delimiter = 's';		/* default delimiter is space */
int32 level = -1;		/* level number */
char *fieldname = NULL;		/* hdfeos field name */
int all_set = FALSE;		/* whether -all option is set */
char objtype = 'N';		/* objtype set to unknowned type first */

FILE *fptr;			/* file pointer */

main(int argc, char *argv[])
{
    int status = 0;
    int32 fid, obj_id;
    char *filename = NULL;
    char *obj_name = NULL;
    char *output_file = NULL;


    /* check command line inputs */
    status = check_options(argc, argv);
    if (status == -1)
	return (-1);


    /* open file pointer (fp) */
    if ((fptr = fopen(argv[argc - 1], "wb+")) == NULL) {
	fprintf(stderr, "Can not open file. \n");
	exit(1);
    }				/* end of fopen */
    /* set filename and object name */
    output_file = argv[argc - 1];	/* binary data output file */
    filename = argv[argc - 2];
    obj_name = argv[argc - 3];


    /* Call EHopen to open hdfeos file  with read only access - if
     * successful the id of the file is returned otherwise a -1 is
     * returned */
    fid = EHopen(filename, DFACC_READ);
    if (fid == -1) {
	fprintf(stderr, "Unable to open file %s.\n", filename);
	return (-1);
    }
    /* determine the object type( G - grid, P - point, S - swath) */
    obj_id = getobjtype(fid, obj_name);
    if (obj_id == -1) {
	fprintf(stderr, "Error encountered in getobjtype(): invalid object %s.\n", obj_name);
	EHclose(fid);
	return (-1);
    }
    /* process the hdfeos object */
    switch (objtype) {
    case 'S':

	status = dumpswath(obj_id, fieldname);
	if (status == -1)
	    fprintf(stderr, "Error encountered in dumpswath().\n");

	SWdetach(obj_id);
	SWclose(fid);
	break;

    case 'G':
	status = dumpgrid(obj_id, fieldname );
	if (status == -1)
	    fprintf(stderr, "Error encountered in dumpgrid().\n");

	GDdetach(obj_id);
	GDclose(fid);
	break;

    case 'P':

	if ((level == -1) && (fieldname != NULL)) {
	    fprintf(stderr, "Invalid command input for point object. You must specify '-l level'\n");
	    return (-1);
	}
	status = dumppoint(obj_id, level, fieldname);
	if (status == -1)
	    fprintf(stderr, "Error encountered in dumppoint().\n");

	PTdetach(obj_id);
	PTclose(fid);

	break;

    }				/* end of switch */
    fclose(fptr);		/* close the file pointer */
    return (status);

}				/* end main */


/********************************************************************
 * FUNCTION : check_options
 * 
 * DESCRIPTION:
 *
 * This function checks the command line inputs entered by the user and
 * set the values for global variable  'col_in_a_line', 'level' 
 * and 'fieldname'.	
 *
 * If an error occurs in this function a value of -1 is returned and the 
 * program writes an error message to stderr.           
 * 
 * INPUTS: 
 *         Name     Type        Description
 *         =====    =====      ===================================
 *         argc     int         No of arguments in command line
 *         argv[]   char *      array of command line arguments
 * 
 * OUTPUTS:
 *         None
 *
 * RETURN VALUE: 
 *        status     int        0   SUCCESSED 
 *                             -1   FAIL
 *
 *******************************************************************/
int check_options(int argc, char *argv[])
{
    int status = 0, i;

    /* Check whether all command line required arguments have been
     * entered: required inputs are: objectname, filename, output file
     * name */
    if (argc < 5) {
	fprintf(stderr, "Insufficient command inputs.\n");
	usage(argv[0]);
	return (-1);

    }
    /* end of 1st if */ 
    else if (argc > 9) {
	fprintf(stderr, "Too many command inputs.\n");
	usage(argv[0]);
	return (-1);

    }
    /* end of 2nd if */

    /* set the global variables */
    for (i = 1; i < argc - 3; i++) {

	if (strcmp(argv[i], "-l") == 0) {
	    if (atoi(argv[i + 1]) < 0 || atoi(argv[i + 1]) > 7) {
		fprintf(stderr, "Invalid command input '-l %s'\n\n", argv[i + 1]);
		usage(argv[0]);
		return (-1);
	    }			
	    /* end of atoi if statment */

	    level = atoi(argv[i + 1]);
	    i++;
	}
	/* end of if -l check */
 
	else if (strcmp(argv[i], "-f") == 0) {
	    fieldname = argv[i + 1];
	    i++;
	}
	/* end of else if 1 */
 
	else if (strcmp(argv[i], "-all") == 0) {
	    all_set = TRUE;
	}
	/* end of else if 2 */
 
	else {
	    fprintf(stderr, "Invalid command input '%s'\n", argv[i]);
	    usage(argv[0]);
	    return (-1);
	}			
	/* end of else */
    }				
    /* end of for loop */

    if ((all_set == TRUE && fieldname != NULL) ||
	(all_set == TRUE && level != -1)) {
	fprintf(stderr, "Invalid command inputs: '-all' can not combine with '-l' or '-f'\n");
	usage(argv[0]);
	return (-1);
    }				
    /* end of if */
    return (status);
}				
/* end of function check option */


/********************************************************************
 * FUNCTION : getobjtype()
 * 
 * DESCRIPTION: identify the object type and return object id 
 *          
 * INPUTS: 
 *         Name     Type        Description
 *         =====    =====      ===================================
 *         fid      int32       file ID from Hopen()
 *         objname  *char       object name
 *
 * OUTPUTS:
 *         objtype  *char       object type
 *
 * RETURN VALUE: 
 *         obj_id    int32      hdfeos object ID, or -1 if object not found.
 *
 ***********************************************************************/
int32 getobjtype(int32 fid, char *objname)
{
    int32 obj_id;

    /* because don't know the object type, try to attach object with three 
     * kinds of interface */

    /* Swath interface */
    obj_id = SWattach(fid, objname);
    if (obj_id != -1)
	objtype = 'S';
    else {
	SWdetach(obj_id);

	/* Point interface */
	obj_id = PTattach(fid, objname);
	if (obj_id != -1)
	    objtype = 'P';
	else {
	    PTdetach(obj_id);

	    /* grid interface */
	    obj_id = GDattach(fid, objname);
	    if (obj_id != -1)
		objtype = 'G';
	    else
		GDdetach(obj_id);
	}			
	/* end of 2nd else */

    }				
    /* end of 1st else */

    return (obj_id);
}				
/* end of getobjtype() */


/*****************************************************************
* FUNCTION : dump_SWGD_field
* 
* DESCRIPTION - This function writes the dimension information and 
*               data to an output file for specified hdfeos swath 
*               or grid objects.
*               
*          Name     Type        Description
*          =====    =====      ===================================
* INPUTS:  obj_id    int32      hdfeos object id
*          fieldname *char      hdfeos fieldname
* 
* OUTPUTS: None
*
* RETURN VALUE:  status         0 successed , -1 failed
*
*******************************************************************/
intn dump_SWGD_field(int32 obj_id, char *fieldname)
{
    int num_of_rows = 0, i = 0;    
    intn status = 0;
    int32 rank =0, ntype =0, num_bytes =0;
    int32 dims[8], bufsize = 1;
    char dimlist[256];
    VOIDP databuf = NULL;

    for (i=0; i < 8; i++) dims[i]=0;
    memset(dimlist, '\0', sizeof(dimlist));

    /* get info for specified field in a swath */
    if (objtype == 'S') {

	status = SWfieldinfo(obj_id, fieldname, &rank, dims, &ntype, dimlist);
	if (status == -1) {
	    fprintf(stderr, "Error encountered in SWfieldinfo()\n");
	    return (-1);

	}			
	/* end if status == -1 */
	for (i = 0; i < rank; i++)
	    bufsize = bufsize * dims[i];

	if (bufsize != 0) {
	    num_of_rows = bufsize / col_in_a_line;
	    databuf = (VOIDP) malloc(DFKNTsize(ntype) * bufsize);

	    if (databuf == NULL) {
		fprintf(stderr, "Error encountered in memory allocation\n");
		return (-1);

	    }			
	    /* end of if(databuf==NULL) */

	    /* read field data */
	    status = SWreadfield(obj_id, fieldname, NULL, NULL, NULL, databuf);
	    if (status == -1) {
		fprintf(stderr, "Error encountered in SWreadfield()\n");
		return (-1);
	    }			
	    /* end of if(status =-1) */
	}			
	/* end of if (bufsize !=0) */
    }
    /* end of 1st if */

    /* get info for specified field in a grid */ 
    else if (objtype == 'G') {

	/* retrieve info about a specified data field or geo field */
	status = GDfieldinfo(obj_id, fieldname, &rank, dims, &ntype, dimlist);
	if (status == -1) {
	    fprintf(stderr, "Error encountered in GDfieldinfo()\n");
	    return (-1);
	}			
	/* end of if(status == -1) */

	for (i = 0; i < rank; i++)
	    bufsize = bufsize * dims[i];

	if (bufsize != 0) {
	    num_of_rows = bufsize / col_in_a_line;
	    databuf = (VOIDP) malloc(DFKNTsize(ntype) * bufsize);

	    if (databuf == NULL) {
		fprintf(stderr, "Error encountered in memory allocation\n");
		return (-1);
	    }			
	    /* end of if(databuf == NULL) */

	    /* read field data */
	    status = GDreadfield(obj_id, fieldname, NULL, NULL, NULL, databuf);
	    if (status == -1) {
		fprintf(stderr, "Error encountered in GDreadfield()\n");
		return (-1);
	    }			
	    /* end of if(status == -1) */
	}			
	/* end of if(bufsize !=0) */
    }				
    /* end of else if(objtype == 'G') */

    /* write field dimension info */
    fwrite(&rank, sizeof(rank), 1, fptr);
    for (i = 0; i < rank; i++) {
	fwrite(&dims[i], sizeof(dims[i]), 1, fptr);
    }				
    /* end of for loop */  
 

    /* write binary data to a file */
    if (bufsize != 0) {
	num_bytes = DFKNTsize(ntype | DFNT_NATIVE);
	fwrite(databuf, (num_bytes * col_in_a_line), num_of_rows, fptr);
	
    }				
    /* end of if(bufsize !=0) */

    if (databuf != NULL)
	free(databuf);
    return (status);
}				
/* end of function dump_SWGD_field */


/********************************************************************
 * FUNCTION : dumpswath
 * 
 * DESCRIPTION: Find the  general Geo and data field information in the swath
 *              and dump the specified field or all fields.
 *          
 * INPUTS: 
 *         Name     Type        Description
 *         =====    =====      ===================================
 *         obj_id   int32       hdfeos object ID 
 *         fieldname  *char     field name or NULL
 *
 * OUTPUTS:
 *         None
 *
 * RETURN VALUE: 
 *         status    int32      0 successful,  -1 if failed
 *
 ***********************************************************************/
intn dumpswath(int32 obj_id, char *fieldname)
{
    intn status = 0, i;
    int32 nflds, *rank, *ntype;
    int32 strbufsize;
    char *fieldlist;
    char fldname[256];

    /* process the specified field */
    if (fieldname != NULL)
	status = dump_SWGD_field(obj_id, fieldname);
    else {			/* process all geo and data fields */

	/* inquire geo field */
	nflds = SWnentries(obj_id, HDFE_NENTGFLD, &strbufsize);
	if (nflds != -1) {
	    rank = (int32 *) calloc(nflds, 4);
	    ntype = (int32 *) calloc(nflds, 4);
	    fieldlist = (char *)calloc(strbufsize + 1, 1);

	    if (rank == NULL || ntype == NULL || fieldlist == NULL) {
		fprintf(stderr, "Error in memory allocation\n");
		return (-1);
	    }			
	    /* end of if(rank == NULL)etc.  */

	    /* retrieve info about geolocation fields defined */
	    SWinqgeofields(obj_id, fieldlist, rank, ntype);
		for(i=0; i<nflds;i++) {
                         if(i==0)
                                 strcpy(fldname, strtok(fieldlist,","));
                         else
                                 strcpy(fldname, strtok(NULL,","));
                         strcat(fldname,"\0");
	    status = dump_SWGD_field(obj_id, fldname);	   
		}		
	    free(rank);
	    free(ntype);
	    free(fieldlist);
	}			
	/* end of if(nflds != -1) for geo field */

	/* retrieve info about datafield defined */
	nflds = SWnentries(obj_id, HDFE_NENTDFLD, &strbufsize);
	if (nflds != -1) {
	    rank = (int32 *) calloc(nflds, 4);
	    ntype = (int32 *) calloc(nflds, 4);
	    fieldlist = (char *)calloc(strbufsize + 1, 1);

	    if (rank == NULL || ntype == NULL || fieldlist == NULL) {
		fprintf(stderr, "Error in memory allocation\n");
		return (-1);
	    }			
	    /* end of if(rank == NULL etc....)*/

	    /* retrieve info about data fields defined */
	    SWinqdatafields(obj_id, fieldlist, rank, ntype);

		/* go through all the data fields and dump them */
		for(i=0; i<nflds;i++) {
                         if(i==0)
                                 strcpy(fldname, strtok(fieldlist,","));
                         else
                                 strcpy(fldname, strtok(NULL,","));
                         strcat(fldname,"\0");

	    status = dump_SWGD_field(obj_id, fldname);
		}
	    free(rank);
	    free(ntype);
	    free(fieldlist);
	}			
	/* end of if(nflds != -1) data field */
    }			        
    /* end of 1st else */

    return (status);
}				
/* end of function dumpswath */


/********************************************************************
 * FUNCTION : dumpgrid
 * 
 * DESCRIPTION: Find the  general projection and data field information in the 
 *              grid and dump the specified field or all fields.
 *          
 * INPUTS: 
 *         Name     Type        Description
 *         =====    =====      ===================================
 *         obj_id   int32       hdfeos object ID 
 *         fieldname  *char     field name or NULL
 *
 * OUTPUTS:
 *         None
 *
 * RETURN VALUE: 
 *         status    int32      0 successful,  -1 if failed
 *
 ***********************************************************************/
intn dumpgrid(int32 obj_id, char *fieldname)
{

    intn status = 0, i;
    int32 xdimsize, ydimsize;
    int32 nflds, *rank, *ntype;
    int32 strbufsize;
    char *fieldlist, fldname[256];
	char buffer[2048];
    int projcode, zonecode, spherecode;
    float64 upleft[2], lowright[2], projparam[16];
    double min_value;

    /* retrieve all GCTP projection info */
    GDprojinfo(obj_id, &projcode, &zonecode, &spherecode, projparam);
    GDgridinfo(obj_id, &xdimsize, &ydimsize, upleft, lowright);

    /* convert the packed format to degree */
    if (projcode == GCTP_GEO) {
	upleft[0] = UnpackDegree(upleft[0]);
	upleft[1] = UnpackDegree(upleft[1]);
	lowright[0] = UnpackDegree(lowright[0]);
	lowright[1] = UnpackDegree(lowright[1]);
    }				
    /* end of if(projcode == GCTP_GEO) */

    /* make sure upleft is upleft */
    if (upleft[0] > lowright[0]) {
	min_value = lowright[0];
	lowright[0] = upleft[0];
	upleft[0] = min_value;

    }				
    /* end of if(upleft[0] > lowright[0]) */

    if (upleft[1] < lowright[1]) {
	min_value = upleft[1];
	upleft[1] = lowright[1];
	lowright[1] = min_value;

    }				
    /* end of if(upleft[1] < lowright[1]) */

    /* get info for specified field */
    if (fieldname != NULL)
	status = dump_SWGD_field(obj_id, fieldname);
    else {

	/* inquire data field */
	nflds = GDnentries(obj_id, HDFE_NENTDFLD, &strbufsize);
	if (nflds != -1) {
	    rank = (int32 *) calloc(nflds, 4); 
	    ntype = (int32 *) calloc(nflds, 4);
	    fieldlist = (char *)calloc(strbufsize + 1, 1);

	    if (rank == NULL || ntype == NULL || fieldlist == NULL) {
		fprintf(stderr, "Error in memory allocation\n");
		return (-1);
	    }			
	    /* end of if (rank ==NULL)etc... */

	    /* retrieve info about data fields defined */
	    GDinqfields(obj_id, fieldlist, rank, ntype);
		for(i=0; i<nflds; i++){
			if(i==0)
				strcpy(fldname,strtok(fieldlist,","));
			else
				strcpy(fldname,strtok(NULL,","));
			strcat(fldname,"\0");
	    status = dump_SWGD_field(obj_id, fldname);	   
		}
	    free(rank);
	    free(ntype);
	    free(fieldlist);

	}			
	/* end of if( nflds != -1 ) */
    }				
    /* end of else */

    return (status);
}				
/* end od dumpgrid() */


/********************************************************************
 * FUNCTION : dumppoint
 * 
 * DESCRIPTION: Find the  general level and data field information in the 
 *              point and dump the specified field in a level or all fields
 *              in a level or all levels.
 *          
 * INPUTS: 
 *         Name     Type        Description
 *         =====    =====      ===================================
 *         obj_id   int32       hdfeos object ID 
 *         level    int32       level number
 *         fieldname  *char     field name or NULL
 *
 * OUTPUTS:
 *         None
 *
 * RETURN VALUE: 
 *         status    int32      0 successful,  -1 if failed
 *
 ***********************************************************************/
intn dumppoint(int32 obj_id, int32 level, char *fieldname)
{
    intn status = 0;
    int32 nlevels;
    int i;

    if (level != -1)
	status = dump_PT_field(obj_id, level, fieldname);

    else {  
      /* get all fields in all levels */

	if ((nlevels = PTnlevels(obj_id)) == -1) {
	    fprintf(stderr, "Error encountered in PTnlevel()\n");
	    return (-1);
	}
	for (i = 0; i < nlevels; i++)
	    status = dump_PT_field(obj_id, i, fieldname);
    }
    return (status);
}				
/* end of dumppoint() */


/*****************************************************************
* FUNCTION : dump_PT_field
* 
* DESCRIPTION - This function writes the level information and
*               binary data to a file for the specified 
*               hdfeos  point object .
*               
*          Name     Type        Description
*          =====    =====      ===================================
* INPUTS:  obj_id    int32      hdfeos object id
*          level     int32      level number
*          fieldname *char      hdfeos fieldname
* 
* OUTPUTS: None
*
* RETURN VALUE:  status         0 successed , -1 failed
*
*******************************************************************/
#define BUFFER 1000000
intn dump_PT_field(int32 obj_id, int32 level, char *fieldname)
{

    intn status = 0, islink;
    int32 lnamesize, listsize, i;
    char *fieldlist, *fields;
    int32 *fldtypes, *fldorders;
    char flinkfield[256], blinkfield[256];
    int32 nfields, nrecs;
    char levelname[100];
    int32 vsize;
    uint32 *bb;
    int32 count;
    int32 *off;
    int32 bufsize;		/* size of the buffer we are using */
    int32 *recs, *fldlevel;


    PTgetlevelname(obj_id, level, levelname, &lnamesize);
    nfields = PTnfields(obj_id, level, &listsize);
    nrecs = PTnrecs(obj_id, level);

    islink = PTfwdlinkinfo(obj_id, level, flinkfield);
    if (islink == -1)
	strcpy(flinkfield, "|!@#$thisisnotafieldname|!@#$");

    islink = PTbcklinkinfo(obj_id, level, blinkfield);
    if (islink == -1)
	strcpy(blinkfield, "|!@#$thisisnotafieldname|!@#$");

    /* make room for and get field stuff */
    fieldlist = (char *)HDmalloc((listsize + 1) * sizeof(char));
    fields = (char *)HDmalloc((listsize + 1) * sizeof(char));

    fldtypes = (int32 *) HDmalloc(nfields * sizeof(int32));
    fldorders = (int32 *) HDmalloc(nfields * sizeof(int32));
    fldlevel = (int32 *) HDmalloc(nfields * sizeof(int32));
    off = (int32 *) HDmalloc(nfields * sizeof(int32));

    if (fieldlist == NULL || fields == NULL || fldtypes == NULL || fldorders == NULL
	|| off == NULL) {
	fprintf(stderr, "Error in memory allocation\n");
	return (-1);

    }
    PTlevelinfo(obj_id, level, fieldlist, fldtypes, fldorders);
    strcpy(fields, fieldlist);

    /* check if it is a valid field name */
    if (fieldname != NULL) {
	if (strstr(fieldlist, fieldname) == NULL) {
	    fprintf(stderr, "Invalid fieldname '%s' in level %d\n", fieldname, level);
	    return (-1);
	}
    }

    /* print level, level name, number of record and number of field */

    if (fieldname == NULL) {
      vsize = PTsizeof(obj_id, fields, fldlevel);
      
    } else {
      vsize = PTsizeof(obj_id, fieldname, fldlevel);	          
    }

    /* print ascii data */

    if (nrecs * vsize > BUFFER) {
      bufsize = BUFFER;
      
    } else {
      bufsize = nrecs * vsize;
    }

    bb = (uint32 *) HDmalloc(bufsize);
   
    if (bb == NULL) {
	fprintf(stderr, "malloc error\n");
	return (-1);
    }

	count = nrecs;
        recs = (int32 *) HDmalloc(count * sizeof(int32));
        for (i = 0; i < nrecs; i++) recs[i]=i;

	/* vsize = Length of each record(It is constant for all records
	 * for eg: 38 for "Point Data") */
	/* nrecs = Number of records */
	
	/* read and update bookkeeping */
	  if (fieldname == NULL) {	  
	  PTreadlevel(obj_id, level, fields, count, recs, bb);
	  fwrite(bb, vsize, nrecs, fptr);/* Write data to a file */	
	  
	} else {
	  PTreadlevel(obj_id, level, fieldname, count, recs, bb);
	  fwrite(bb, vsize, nrecs, fptr); /* Write data to a file */	
	}

    HDfree(bb);
    HDfree(fldtypes);
    HDfree(fldorders);
    HDfree(fieldlist);
    HDfree(fields);
    HDfree(fldlevel);
    HDfree(off);

    return (status);

} /* end of function */ 


/*---------------------------------------------------------------------------
 NAME
       ProjName
 DESCRIPTION
        The function will return projection name for a given code. 
 RETURNS
        projection name in ASCII.
 Author: Dr. Liping Di, Hughes STX, 7701 Greenbelt Road, Suite 400, Greenbelt, MD 20770.

 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for non-commercial use without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

----------------------------------------------------------------------------*/
char *ProjName(int32 projcode)
{
    char *proj;

    switch (projcode) {
    case 0:
	proj = "Geographic";
	break;
    case 1:
	proj = "Universal Transverse Mercator";
	break;
    case 4:
	proj = "Lambert Conformal Conic";
	break;
    case 6:
	proj = "Polar Stereographic";
	break;
    case 7:
	proj = "Polyconic";
	break;
    case 9:
	proj = "Transverse Mercator";
	break;
    case 11:
	proj = "Lambert Azimuthal Equal Area";
	break;
    case 20:
	proj = "Hotine Oblique Mercator";
	break;
    case 22:
	proj = "Space Oblique Mercator";
	break;
    case 24:
	proj = "Interrupted Goode Homolosine";
	break;
    case 99:
	proj = "Intergerized Sinusoidal Projection";
	break;
    default:
	proj = "Unknown projection";
    }

    return proj;
}


/*---------------------------------------------------------------------------
 NAME
       SpheroidName
 DESCRIPTION
        The function will return the spheroid name for a given code. 
 RETURNS
        projection name in ASCII.
 Author: Dr. Liping Di, Hughes STX, 7701 Greenbelt Road, Suite 400, Greenbelt, MD 20770.

 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for non-commercial use without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

----------------------------------------------------------------------------*/
char *SpheroidName(int32 code)
{
    char *sphere[20] =
    {"Clarke 1866",
     "Clarke 1880",
     "Bessel",
     "International 1967",
     "International 1909",
     "WGS 72",
     "Everest",
     "WGS 66",
     "GRS 1980",
     "Airy",
     "Modified Airy",
     "Modified Everest",
     "WGS 84",
     "Southeast Asia",
     "Austrailian National",
     "Krassovsky",
     "Hough",
     "Mercury 1960",
     "Modified Mercury 1968",
     "Sphere of Radius 6370997 m"};

    if (code < 0 || code > 19)
	return "Unknow Spheroid";
    return sphere[code];
}

/*---------------------------------------------------------------------------
 NAME
       ProjParam
 DESCRIPTION
        The function will convert projection parameters into ASCII text. 
 RETURNS
        void.
 Author: Dr. Liping Di, Hughes STX, 7701 Greenbelt Road, Suite 400, Greenbelt, MD 20770.
 Comments: no error checking is perfomed.

 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for non-commercial use without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

----------------------------------------------------------------------------*/
void ProjParam(int32 projcode, float64 param[], int32 zonecode, char *buffer)
{
    switch (projcode) {
    case 1:			/* "Universal Transverse Mercator" */
	if (param[0] == 0.0)
	    sprintf(buffer, "UTM Zone = %d", zonecode);
	else
	    sprintf(buffer, "UTM Origin = %f degree longitude, %f degree latitude", param[0], param[1]);
	break;

    case 4:			/* "Lambert Conformal Conic" */
	sprintf(buffer, "SMajor = %f (m), Sminor = %f (m), STDPR1 = %f, STDPR2 = %f, CentMer = %f",
		param[0], param[1], param[2], param[3], param[4]);
	sprintf(buffer + strlen(buffer), "OriginLat = %f, False East = %f, False North = %f",
		param[5], param[6], param[7]);
	break;

    case 6:			/* "Polar Stereographic" */
	sprintf(buffer, "SMajor=%f, SMinor=%f, LongPol=%f, Latitude of true scale=%f, False East=%f, False North=%f",
	     param[0], param[1], param[4], param[5], param[6], param[7]);
	break;

    case 7:			/* "Polyconic" */
	sprintf(buffer, "SMajor=%f, SMinor=%f, CentMer=%f, OriginLat=%f,    False East=%f, False North=%f",
	     param[0], param[1], param[4], param[5], param[6], param[7]);
	break;

    case 9:			/* "Transverse Mercator" */
	sprintf(buffer, "SMajor=%f, SMinor=%f, Scale Factor=%f, CentMer=%f, OriginLat=%f, False East=%f, False North=%f",
		param[0], param[1], param[2], param[4], param[5], param[6], param[7]);
	break;

    case 11:			/* "Lambert Azimuthal Equal Area" */
	if (param[0] == 0.0)
	    sprintf(buffer, "Radius of reference sphere=6370997");
	else
	    sprintf(buffer, "Radius of reference sphere=%f", param[0]);
	sprintf(buffer + strlen(buffer), "CentLon=%f, CenterLat=%f, False East=%f, False North=%f",
		param[4], param[5], param[6], param[7]);
	break;

    case 20:			/* "Hotine Oblique Mercator-A, B" */
	if (param[12] == 0) {
	    sprintf(buffer, "SMajor=%f, SMinor=%f, Scale Factor=%f, OriginLat=%f, False East=%f, False North=%f",
	     param[0], param[1], param[2], param[5], param[6], param[7]);
	    sprintf(buffer + strlen(buffer), " Longitude of first point on center line=%f,", param[8]);
	    sprintf(buffer + strlen(buffer), " Latitude of first point on center line=%f,", param[9]);
	    sprintf(buffer + strlen(buffer), " Longitude of second point on center line=%f,", param[10]);
	    sprintf(buffer + strlen(buffer), " Latitude of second point on center line=%f,", param[11]);
	} else {
	    sprintf(buffer, "SMajor=%f, SMinor=%f, Scale Factor=%f,", param[0], param[1], param[2]);
	    sprintf(buffer + strlen(buffer), " Azimuth Angle east of north of center line=%f,", param[3]);
	    sprintf(buffer + strlen(buffer), " Longitude of point on central meridian where azimuth occurs=%f,", param[4]);
	    sprintf(buffer + strlen(buffer), " OriginLat=%f, False East=%f, False North=%f",
		    param[5], param[6], param[7]);
	}
	break;

    case 22:			/* Space Oblique Mercator-A, B */
	if (param[12] == 0) {
	    sprintf(buffer, "SMajor=%f, SMinor=%f, Inclination Angle=%f,",
		    param[0], param[1], param[2]);
	    sprintf(buffer + strlen(buffer), " Logitude of ascending orbit at equator=%f, False East=%f, False North=%f,",
		    param[4], param[6], param[7]);
	    sprintf(buffer + strlen(buffer), " Period of satellite revolution in minutes=%f,", param[8]);
	    sprintf(buffer + strlen(buffer), " Satellite ratio=%f,", param[9]);
	    sprintf(buffer + strlen(buffer), " End of path flag=%f,", param[10]);
	} else {
	    sprintf(buffer, "SMajor=%f, SMinor=%f,", param[0], param[1]);
	    sprintf(buffer + strlen(buffer), " Landsat number=%d,", (int)param[2]);
	    sprintf(buffer + strlen(buffer), " Landsat path number=%d,", (int)param[3]);
	    sprintf(buffer + strlen(buffer), " False East=%f, False North=%f", param[6], param[7]);
	}
	break;

    case 24:			/* Interrupted Goode Homolosine */
	if (param[0] == 0)
	    sprintf(buffer, "Radius of reference sphere=6370997");
	else
	    sprintf(buffer, "Radius of reference sphere=%f", param[0]);
	break;

    case 99:			/* Intergerized Sinusoidal Projection */
	if (param[0] == 0)
	    sprintf(buffer, "Radius of reference sphere=6370997,");
	else
	    sprintf(buffer, "Radius of reference sphere=%f,", param[0]);

	sprintf(buffer + strlen(buffer), " Central Meridian=%f,", param[4]);
	sprintf(buffer + strlen(buffer), " Number of equally spaced latitudinal zone(rows)=%d, ",
		(int)param[8]);
	sprintf(buffer + strlen(buffer), " Right justify columns flag=%d, ", (int)param[10]);
	break;

    default:
	sprintf(buffer + strlen(buffer), " Un-supported Projection");
    }

    return;
}

void projcode2text(int32 projcode, int32 zonecode,
		   int32 spherecode, double projparam[], char *buffer)
{
    char *pointer;

    /* get the project name */
    strcpy(buffer, ProjName(projcode));
    if (projcode == 0 || strstr(buffer, "Unknown"))
	return;

    /* get the spheriod name */
    strcat(buffer, "; Spheriod: ");
    pointer = SpheroidName(spherecode);
    strcat(buffer, pointer);

    /* handle the projection paramters */
    strcat(buffer, "; Projection parameters: ");
    ProjParam(projcode, projparam, zonecode, buffer + strlen(buffer));
    return;
}

double UnpackDegree(double packed)
{
    int degree, minute;
    double second, unpacked;

    degree = ((int)packed) / 1000000;
    minute = ((int)packed) / 1000 - degree * 1000;
    second = packed - degree * 1000000 - minute * 1000;

    unpacked = second / 3600 + minute / 60 + degree;
    return unpacked;
}


/*------------------------------------------------------------------ 
 NAME
     get_type
 DESCRIPTION
     Used to get string descritpion of HDF number type.
 RETURNS
     Return a string description of the given number type
     if successful else return default string.
 
 Author: Dr. Liping Di, Hughes STX, 7701 Greenbelt Road, Suite 400, Greenbelt, MD 20770.
 Comments: no error checking is perfomed.

 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its 
 documentation for non-commercial use without fee is hereby granted, provided 
 that the above copyright notice appear in all copies and that both that 
 copyright notice and this permission notice appear in supporting 
 documentation.

----------------------------------------------------------------------------*/
char *get_type(int32 nt /* Number type */ )
{    /* Switch on number type "nt" */
    switch (nt) {
    case DFNT_CHAR:
	return ("8-bit characters");
	break;
    case DFNT_INT8:
	return ("signed 8-bit integers");
	break;
    case DFNT_UINT8:
	return ("unsigned 8-bit integers");
	break;
    case DFNT_INT16:
	return ("signed 16-bit integers");
	break;
    case DFNT_UINT16:
	return ("unsigned 16-bit integers");
	break;
    case DFNT_INT32:
	return ("signed 32-bit integers");
	break;
    case DFNT_UINT32:
	return ("unsigned 32-bit integers");
	break;
    case DFNT_FLOAT32:
	return ("32-bit floating point numbers");
	break;
    case DFNT_FLOAT64:
	return ("64-bit floating point numbers");
	break;
    case DFNT_NCHAR:
	return ("native 8-bit characters");
	break;
    case DFNT_NINT8:
	return ("native signed 8-bit integers");
	break;
    case DFNT_NUINT8:
	return ("native unsigned 8-bit integers");
	break;
    case DFNT_NINT16:
	return ("native signed 16-bit integers");
	break;
    case DFNT_NUINT16:
	return ("native unsigned 16-bit integers");
	break;
    case DFNT_NINT32:
	return ("native signed 32-bit integers");
	break;
    case DFNT_NUINT32:
	return ("native unsigned 32-bit integers");
	break;
    case DFNT_NFLOAT32:
	return ("native 32-bit floating point numbers");
	break;
    case DFNT_NFLOAT64:
	return ("native 64-bit floating point numbers");
	break;
    case DFNT_LCHAR:
	return ("little-endian 8-bit characters");
	break;
    case DFNT_LINT8:
	return ("little-endian signed 8-bit integers");
	break;
    case DFNT_LUINT8:
	return ("little-endian unsigned 8-bit integers");
	break;
    case DFNT_LINT16:
	return ("little-endian signed 16-bit integers");
	break;
    case DFNT_LUINT16:
	return ("little-endian unsigned 16-bit integers");
	break;
    case DFNT_LINT32:
	return ("little-endian signed 32-bit integers");
	break;
    case DFNT_LUINT32:
	return ("little-endian unsigned 32-bit integers");
	break;
    case DFNT_LFLOAT32:
	return ("little-endian 32-bit floating point numbers");
	break;
    case DFNT_LFLOAT64:
	return ("little-endian 64-bit floating point numbers");
	break;
    default:
	return ("unknown number type");
    } /* switch */
} /* get_type */



void usage(char *command)
{
    printf("Usage:\n");
    printf("%s <([-l level] [-f field]) | [-all]>  <objname> <filename> <outputfile> \n\n", command);
    printf("	Required arguments are <objname> and <filename> and <outputfile>.\n\n");
    printf("	 Optional arguments are -l and -f or -all\n\n");
    printf("	-l level:    the level number for point object \n");
    printf("	-f fldname:  the field for hdfeos object.\n");
    printf("        -all:        dump all fields for hdfeos object.\n");
    return;
}
